@@ -533,8 +533,7 @@ xsltAttributeInternal(xsltTransformContextPtr ctxt, xmlNodePtr node,
     if (ctxt->insert == NULL)
         return;
     if (comp == NULL) {
-        xsltPrintErrorContext(ctxt, NULL, inst);
-        xsltGenericError(xsltGenericErrorContext,
+        xsltTransformError(ctxt, NULL, inst,
                          "xsl:attribute : compilation failed\n");
         return;
     }
@@ -546,8 +545,7 @@ xsltAttributeInternal(xsltTransformContextPtr ctxt, xmlNodePtr node,
         return;
     }
     if (ctxt->insert->children != NULL) {
-        xsltPrintErrorContext(ctxt, NULL, inst);
-        xsltGenericError(xsltGenericErrorContext,
+        xsltTransformError(ctxt, NULL, inst,
                          "xsl:attribute : node already has children\n");
         return;
     }
@@ -562,8 +560,7 @@ xsltAttributeInternal(xsltTransformContextPtr ctxt, xmlNodePtr node,
             xsltEvalAttrValueTemplate(ctxt, inst, (const xmlChar *) "name",
                                       XSLT_NAMESPACE);
         if (prop == NULL) {
-            xsltPrintErrorContext(ctxt, NULL, inst);
-            xsltGenericError(xsltGenericErrorContext,
+            xsltTransformError(ctxt, NULL, inst,
                              "xsl:attribute : name is missing\n");
             goto error;
         }
@@ -597,9 +594,8 @@ xsltAttributeInternal(xsltTransformContextPtr ctxt, xmlNodePtr node,
             if (prefix != NULL) {
                 ns = xmlSearchNs(inst->doc, inst, prefix);
                 if (ns == NULL) {
-                    xsltPrintErrorContext(ctxt, NULL, inst);
-                    xsltGenericError(xsltGenericErrorContext,
-                                     "xsl:attribute : no namespace bound to prefix %s\n",
+                    xsltTransformError(ctxt, NULL, inst,
+			 "xsl:attribute : no namespace bound to prefix %s\n",
                                      prefix);
                 } else {
                     ns = xsltGetNamespace(ctxt, inst, ns, ctxt->insert);
@@ -619,22 +615,26 @@ xsltAttributeInternal(xsltTransformContextPtr ctxt, xmlNodePtr node,
 
     if ((fromset) && (ns != NULL))
         URL = ns->href;
-    if ((fromset == 0) || (!xmlHasNsProp(ctxt->insert, name, URL))) {
-        value = xsltEvalTemplateString(ctxt, node, inst);
-        if (value == NULL) {
-            if (ns) {
-                attr = xmlSetNsProp(ctxt->insert, ns, name,
-                                    (const xmlChar *) "");
-            } else {
-                attr =
-                    xmlSetProp(ctxt->insert, name, (const xmlChar *) "");
-            }
+
+    if (fromset) {
+	attr = xmlHasNsProp(ctxt->insert, name, ns);
+	if (attr != NULL)
+	    return;
+    }
+    value = xsltEvalTemplateString(ctxt, node, inst);
+    if (value == NULL) {
+        if (ns) {
+            attr = xmlSetNsProp(ctxt->insert, ns, name,
+                                (const xmlChar *) "");
         } else {
-            if (ns) {
-                attr = xmlSetNsProp(ctxt->insert, ns, name, value);
-            } else {
-                attr = xmlSetProp(ctxt->insert, name, value);
-            }
+            attr =
+                xmlSetProp(ctxt->insert, name, (const xmlChar *) "");
+        }
+    } else {
+        if (ns) {
+            attr = xmlSetNsProp(ctxt->insert, ns, name, value);
+        } else {
+            attr = xmlSetProp(ctxt->insert, name, value);
         }
     }
 
@@ -713,10 +713,11 @@ xsltApplyAttributeSet(xsltTransformContextPtr ctxt, xmlNodePtr node,
 
             style = ctxt->style;
 #ifdef WITH_DEBUGGER
-            if (style && (xslDebugStatus != XSLT_DEBUG_NONE)) {
+            if ((style != NULL) && (style->attributeSets != NULL) &&
+		(xslDebugStatus != XSLT_DEBUG_NONE)) {
                 values =
                     xmlHashLookup2(style->attributeSets, ncname, prefix);
-                if (values)
+                if ((values != NULL) && (values->attr != NULL))
                     xslHandleDebugger(values->attr->parent, node, NULL,
                                       ctxt);
             }
