@@ -10,6 +10,7 @@
  * Bjorn Reese <breese@users.sourceforge.net> for number formatting
  */
 
+#define IN_LIBXSLT
 #include "libxslt.h"
 
 #include <string.h>
@@ -51,6 +52,45 @@
  */
 #define DOCBOOK_XSL_HACK
 
+/**
+ * xsltXPathFunctionLookup:
+ * @ctxt:  a void * but the XSLT transformation context actually
+ * @name:  the function name
+ * @ns_uri:  the function namespace URI
+ *
+ * This is the entry point when a function is needed by the XPath
+ * interpretor.
+ *
+ * Returns the callback function or NULL if not found
+ */
+xmlXPathFunction
+xsltXPathFunctionLookup (xmlXPathContextPtr ctxt,
+			 const xmlChar *name, const xmlChar *ns_uri) {
+    xmlXPathFunction ret;
+
+    if ((ctxt == NULL) || (name == NULL) || (ns_uri == NULL))
+	return (NULL);
+
+#ifdef WITH_XSLT_DEBUG_FUNCTION
+    xsltGenericDebug(xsltGenericDebugContext,
+            "Lookup function {%s}%s\n", ns_uri, name);
+#endif
+
+    /* give priority to context-level functions */
+    ret = (xmlXPathFunction) xmlHashLookup2(ctxt->funcHash, name, ns_uri);
+
+    if (ret == NULL)
+	ret = xsltExtModuleFunctionLookup(name, ns_uri);
+
+#ifdef WITH_XSLT_DEBUG_FUNCTION
+    if (ret != NULL)
+        xsltGenericDebug(xsltGenericDebugContext,
+            "found function %s\n", name);
+#endif
+    return(ret);
+}
+
+
 /************************************************************************
  *									*
  *			Module interfaces				*
@@ -66,146 +106,145 @@
  *   node-set document(object, node-set?)
  */
 void
-xsltDocumentFunction(xmlXPathParserContextPtr ctxt, int nargs){
+xsltDocumentFunction(xmlXPathParserContextPtr ctxt, int nargs)
+{
     xsltDocumentPtr doc;
     xmlXPathObjectPtr obj, obj2 = NULL;
     xmlChar *base = NULL, *URI;
 
 
     if ((nargs < 1) || (nargs > 2)) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
-		"document() : invalid number of args %d\n", nargs);
-	ctxt->error = XPATH_INVALID_ARITY;
-	return;
+        xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
+                         "document() : invalid number of args %d\n",
+                         nargs);
+        ctxt->error = XPATH_INVALID_ARITY;
+        return;
     }
     if (ctxt->value == NULL) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
-	    "document() : invalid arg value\n");
-	ctxt->error = XPATH_INVALID_TYPE;
-	return;
+        xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
+                         "document() : invalid arg value\n");
+        ctxt->error = XPATH_INVALID_TYPE;
+        return;
     }
 
     if (nargs == 2) {
-	if (ctxt->value->type != XPATH_NODESET) {
-	    xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt),
-		                  NULL, NULL);
-	    xsltGenericError(xsltGenericErrorContext,
-		"document() : invalid arg expecting a nodeset\n");
-	    ctxt->error = XPATH_INVALID_TYPE;
-	    return;
-	}
+        if (ctxt->value->type != XPATH_NODESET) {
+            xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
+                             "document() : invalid arg expecting a nodeset\n");
+            ctxt->error = XPATH_INVALID_TYPE;
+            return;
+        }
 
-	obj2 = valuePop(ctxt);
+        obj2 = valuePop(ctxt);
     }
 
     if (ctxt->value->type == XPATH_NODESET) {
-	int i;
-	xmlXPathObjectPtr newobj, ret;
+        int i;
+        xmlXPathObjectPtr newobj, ret;
 
-	obj = valuePop(ctxt);
-	ret = xmlXPathNewNodeSet(NULL);
+        obj = valuePop(ctxt);
+        ret = xmlXPathNewNodeSet(NULL);
 
-	if (obj->nodesetval) {
-	    for (i = 0; i < obj->nodesetval->nodeNr; i++) {
-		valuePush(ctxt,
-			  xmlXPathNewNodeSet(obj->nodesetval->nodeTab[i]));
-		xmlXPathStringFunction(ctxt, 1);
-		if (nargs == 2) {
-		    valuePush(ctxt, xmlXPathObjectCopy(obj2));
-		} else {
-		    valuePush(ctxt,
-			      xmlXPathNewNodeSet(obj->nodesetval->nodeTab[i]));
-		}
-		xsltDocumentFunction(ctxt, 2);
-		newobj = valuePop(ctxt);
-		ret->nodesetval = xmlXPathNodeSetMerge(ret->nodesetval,
-						       newobj->nodesetval);
-		xmlXPathFreeObject(newobj);
-	    }
-	}
+        if (obj->nodesetval) {
+            for (i = 0; i < obj->nodesetval->nodeNr; i++) {
+                valuePush(ctxt,
+                          xmlXPathNewNodeSet(obj->nodesetval->nodeTab[i]));
+                xmlXPathStringFunction(ctxt, 1);
+                if (nargs == 2) {
+                    valuePush(ctxt, xmlXPathObjectCopy(obj2));
+                } else {
+                    valuePush(ctxt,
+                              xmlXPathNewNodeSet(obj->nodesetval->
+                                                 nodeTab[i]));
+                }
+                xsltDocumentFunction(ctxt, 2);
+                newobj = valuePop(ctxt);
+                ret->nodesetval = xmlXPathNodeSetMerge(ret->nodesetval,
+                                                       newobj->nodesetval);
+                xmlXPathFreeObject(newobj);
+            }
+        }
 
-	xmlXPathFreeObject(obj);
-	if (obj2 != NULL)
-	    xmlXPathFreeObject(obj2);
-	valuePush(ctxt, ret);
-	return;
+        xmlXPathFreeObject(obj);
+        if (obj2 != NULL)
+            xmlXPathFreeObject(obj2);
+        valuePush(ctxt, ret);
+        return;
     }
     /*
      * Make sure it's converted to a string
      */
     xmlXPathStringFunction(ctxt, 1);
     if (ctxt->value->type != XPATH_STRING) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
-	    "document() : invalid arg expecting a string\n");
-	ctxt->error = XPATH_INVALID_TYPE;
-	if (obj2 != NULL)
-	    xmlXPathFreeObject(obj2);
-	return;
+        xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
+                         "document() : invalid arg expecting a string\n");
+        ctxt->error = XPATH_INVALID_TYPE;
+        if (obj2 != NULL)
+            xmlXPathFreeObject(obj2);
+        return;
     }
     obj = valuePop(ctxt);
     if (obj->stringval == NULL) {
-	valuePush(ctxt, xmlXPathNewNodeSet(NULL));
+        valuePush(ctxt, xmlXPathNewNodeSet(NULL));
     } else {
-	if ((obj2 != NULL) && (obj2->nodesetval != NULL) &&
-	      (obj2->nodesetval->nodeNr > 0)) {
-	    xmlNodePtr target;
+        if ((obj2 != NULL) && (obj2->nodesetval != NULL) &&
+            (obj2->nodesetval->nodeNr > 0) &&
+            IS_XSLT_REAL_NODE(obj2->nodesetval->nodeTab[0])) {
+            xmlNodePtr target;
 
-	    target = obj2->nodesetval->nodeTab[0];
-	    if (target->type == XML_ATTRIBUTE_NODE) {
-		target = ((xmlAttrPtr) target)->parent;
-	    }
-	    base = xmlNodeGetBase(target->doc, target);
-	} else {
-	    xsltTransformContextPtr tctxt;
-	    
-	    tctxt = xsltXPathGetTransformContext(ctxt);
-	    if ((tctxt != NULL) && (tctxt->inst != NULL)) {
-		base = xmlNodeGetBase(tctxt->inst->doc, tctxt->inst);
-	    } else if ((tctxt != NULL) && (tctxt->style != NULL) &&
-		       (tctxt->style->doc != NULL)) {
-		base = xmlNodeGetBase(tctxt->style->doc, 
-			              (xmlNodePtr) tctxt->style->doc);
-	    }
-	}
-	URI = xmlBuildURI(obj->stringval, base);
-	if (base != NULL)
-	    xmlFree(base);
+            target = obj2->nodesetval->nodeTab[0];
+            if (target->type == XML_ATTRIBUTE_NODE) {
+                target = ((xmlAttrPtr) target)->parent;
+            }
+            base = xmlNodeGetBase(target->doc, target);
+        } else {
+            xsltTransformContextPtr tctxt;
+
+            tctxt = xsltXPathGetTransformContext(ctxt);
+            if ((tctxt != NULL) && (tctxt->inst != NULL)) {
+                base = xmlNodeGetBase(tctxt->inst->doc, tctxt->inst);
+            } else if ((tctxt != NULL) && (tctxt->style != NULL) &&
+                       (tctxt->style->doc != NULL)) {
+                base = xmlNodeGetBase(tctxt->style->doc,
+                                      (xmlNodePtr) tctxt->style->doc);
+            }
+        }
+        URI = xmlBuildURI(obj->stringval, base);
+        if (base != NULL)
+            xmlFree(base);
         if (URI == NULL) {
-	    valuePush(ctxt, xmlXPathNewNodeSet(NULL));
-	} else {
-	    xsltTransformContextPtr tctxt;
+            valuePush(ctxt, xmlXPathNewNodeSet(NULL));
+        } else {
+            xsltTransformContextPtr tctxt;
 
-	    tctxt = xsltXPathGetTransformContext(ctxt);
-	    if (tctxt == NULL) {
-		xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt),
-			              NULL, NULL);
-		xsltGenericError(xsltGenericErrorContext,
-			"document() : internal error tctxt == NULL\n");
-		valuePush(ctxt, xmlXPathNewNodeSet(NULL));
-	    } else {
-                if (xmlStrEqual(tctxt->style->doc->URL, URI)) {
-                    valuePush(ctxt, xmlXPathNewNodeSet((xmlNodePtr)tctxt->style->doc));
-                }
-                else {
-		    doc = xsltLoadDocument(tctxt, URI);
-		    if (doc == NULL)
-		        valuePush(ctxt, xmlXPathNewNodeSet(NULL));
-		    else {
-		        /* TODO: use XPointer of HTML location for fragment ID */
-		        /* pbm #xxx can lead to location sets, not nodesets :-) */
-		        valuePush(ctxt, xmlXPathNewNodeSet((xmlNodePtr) doc->doc));
+            tctxt = xsltXPathGetTransformContext(ctxt);
+            if (tctxt == NULL) {
+                xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
+                                 "document() : internal error tctxt == NULL\n");
+                valuePush(ctxt, xmlXPathNewNodeSet(NULL));
+            } else {
+                doc = xsltLoadDocument(tctxt, URI);
+                if (doc == NULL) {
+		    if ((xmlStrEqual(URI, BAD_CAST "")) ||
+			(xmlStrEqual(tctxt->style->doc->URL, URI))) {
+			valuePush(ctxt, xmlXPathNewNodeSet(
+				    (xmlNodePtr)tctxt->style->doc));
+		    } else {
+			valuePush(ctxt, xmlXPathNewNodeSet(NULL));
 		    }
+		} else {
+                    /* TODO: use XPointer of HTML location for fragment ID */
+                    /* pbm #xxx can lead to location sets, not nodesets :-) */
+                    valuePush(ctxt,
+                              xmlXPathNewNodeSet((xmlNodePtr) doc->doc));
                 }
-	    }
-	    xmlFree(URI);
-	}
+            }
+            xmlFree(URI);
+        }
     }
     xmlXPathFreeObject(obj);
     if (obj2 != NULL)
-	xmlXPathFreeObject(obj2);
+        xmlXPathFreeObject(obj2);
 }
 
 /**
@@ -225,8 +264,7 @@ xsltKeyFunction(xmlXPathParserContextPtr ctxt, int nargs){
     xsltTransformContextPtr tctxt;
 
     if (nargs != 2) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"key() : expects two arguments\n");
 	ctxt->error = XPATH_INVALID_ARITY;
 	return;
@@ -235,8 +273,7 @@ xsltKeyFunction(xmlXPathParserContextPtr ctxt, int nargs){
     obj2 = valuePop(ctxt);
     if ((obj2 == NULL) ||
 	(ctxt->value == NULL) || (ctxt->value->type != XPATH_STRING)) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 	    "key() : invalid arg expecting a string\n");
 	ctxt->error = XPATH_INVALID_TYPE;
 	xmlXPathFreeObject(obj2);
@@ -282,9 +319,7 @@ xsltKeyFunction(xmlXPathParserContextPtr ctxt, int nargs){
 	    if (prefix != NULL) {
 		keyURI = xmlXPathNsLookup(ctxt->context, prefix);
 		if (keyURI == NULL) {
-		    xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt),
-					  NULL, NULL);
-		    xsltGenericError(xsltGenericErrorContext,
+		    xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 			"key() : prefix %s is not bound\n", prefix);
 		}
 		xmlFree(prefix);
@@ -299,9 +334,7 @@ xsltKeyFunction(xmlXPathParserContextPtr ctxt, int nargs){
 	valuePush(ctxt, obj2);
 	xmlXPathStringFunction(ctxt, 1);
 	if ((ctxt->value == NULL) || (ctxt->value->type != XPATH_STRING)) {
-	    xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt),
-				  NULL, NULL);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"key() : invalid arg expecting a string\n");
 	    ctxt->error = XPATH_INVALID_TYPE;
 	    xmlXPathFreeObject(obj1);
@@ -449,9 +482,7 @@ xsltGenerateIdFunction(xmlXPathParserContextPtr ctxt, int nargs){
 
 	if ((ctxt->value == NULL) || (ctxt->value->type != XPATH_NODESET)) {
 	    ctxt->error = XPATH_INVALID_TYPE;
-	    xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt),
-				  NULL, NULL);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"generate-id() : invalid arg expecting a node-set\n");
 	    return;
 	}
@@ -470,8 +501,7 @@ xsltGenerateIdFunction(xmlXPathParserContextPtr ctxt, int nargs){
 	}
 	xmlXPathFreeObject(obj);
     } else {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"generate-id() : invalid number of args %d\n", nargs);
 	ctxt->error = XPATH_INVALID_ARITY;
 	return;
@@ -501,15 +531,13 @@ xsltSystemPropertyFunction(xmlXPathParserContextPtr ctxt, int nargs){
     const xmlChar *nsURI = NULL;
 
     if (nargs != 1) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"system-property() : expects one string arg\n");
 	ctxt->error = XPATH_INVALID_ARITY;
 	return;
     }
     if ((ctxt->value == NULL) || (ctxt->value->type != XPATH_STRING)) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 	    "system-property() : invalid arg expecting a string\n");
 	ctxt->error = XPATH_INVALID_TYPE;
 	return;
@@ -524,9 +552,7 @@ xsltSystemPropertyFunction(xmlXPathParserContextPtr ctxt, int nargs){
 	} else {
 	    nsURI = xmlXPathNsLookup(ctxt->context, prefix);
 	    if (nsURI == NULL) {
-		xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt),
-			              NULL, NULL);
-		xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		    "system-property() : prefix %s is not bound\n", prefix);
 	    }
 	}
@@ -598,15 +624,13 @@ xsltElementAvailableFunction(xmlXPathParserContextPtr ctxt, int nargs){
     xsltTransformContextPtr tctxt;
 
     if (nargs != 1) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"element-available() : expects one string arg\n");
 	ctxt->error = XPATH_INVALID_ARITY;
 	return;
     }
     if ((ctxt->value == NULL) || (ctxt->value->type != XPATH_STRING)) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 	    "element-available() : invalid arg expecting a string\n");
 	ctxt->error = XPATH_INVALID_TYPE;
 	return;
@@ -614,8 +638,7 @@ xsltElementAvailableFunction(xmlXPathParserContextPtr ctxt, int nargs){
     obj = valuePop(ctxt);
     tctxt = xsltXPathGetTransformContext(ctxt);
     if (tctxt == NULL) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"element-available() : internal error tctxt == NULL\n");
 	xmlXPathFreeObject(obj);
 	valuePush(ctxt, xmlXPathNewBoolean(0));
@@ -625,18 +648,20 @@ xsltElementAvailableFunction(xmlXPathParserContextPtr ctxt, int nargs){
 
     name = xmlSplitQName2(obj->stringval, &prefix);
     if (name == NULL) {
+	xmlNsPtr ns;
+
 	name = xmlStrdup(obj->stringval);
+	ns = xmlSearchNs(tctxt->inst->doc, tctxt->inst, NULL);
+	if (ns != NULL) nsURI = xmlStrdup(ns->href);
     } else {
 	nsURI = xmlXPathNsLookup(ctxt->context, prefix);
 	if (nsURI == NULL) {
-	    xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt),
-				  NULL, NULL);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"element-available() : prefix %s is not bound\n", prefix);
 	}
     }
 
-    if (xmlHashLookup2(tctxt->extElements, name, nsURI) != NULL) {
+    if (xsltExtElementLookup(tctxt, name, nsURI) != NULL) {
 	valuePush(ctxt, xmlXPathNewBoolean(1));
     } else {
 	valuePush(ctxt, xmlXPathNewBoolean(0));
@@ -664,15 +689,13 @@ xsltFunctionAvailableFunction(xmlXPathParserContextPtr ctxt, int nargs){
     const xmlChar *nsURI = NULL;
 
     if (nargs != 1) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"function-available() : expects one string arg\n");
 	ctxt->error = XPATH_INVALID_ARITY;
 	return;
     }
     if ((ctxt->value == NULL) || (ctxt->value->type != XPATH_STRING)) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 	    "function-available() : invalid arg expecting a string\n");
 	ctxt->error = XPATH_INVALID_TYPE;
 	return;
@@ -685,9 +708,7 @@ xsltFunctionAvailableFunction(xmlXPathParserContextPtr ctxt, int nargs){
     } else {
 	nsURI = xmlXPathNsLookup(ctxt->context, prefix);
 	if (nsURI == NULL) {
-	    xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt),
-				  NULL, NULL);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"function-available() : prefix %s is not bound\n", prefix);
 	}
     }
@@ -718,16 +739,14 @@ xsltCurrentFunction(xmlXPathParserContextPtr ctxt, int nargs){
     xsltTransformContextPtr tctxt;
 
     if (nargs != 0) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"current() : function uses no argument\n");
 	ctxt->error = XPATH_INVALID_ARITY;
 	return;
     }
     tctxt = xsltXPathGetTransformContext(ctxt);
     if (tctxt == NULL) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"current() : internal error tctxt == NULL\n");
 	valuePush(ctxt, xmlXPathNewNodeSet(NULL));
     } else {
@@ -735,171 +754,6 @@ xsltCurrentFunction(xmlXPathParserContextPtr ctxt, int nargs){
     }
 }
 
-/************************************************************************
- * 									*
- * 		Test of the extension module API			*
- * 									*
- ************************************************************************/
-
-static xmlChar *testData = NULL;
-
-/**
- * xsltExtFunctionTest:
- * @ctxt:  the XPath Parser context
- * @nargs:  the number of arguments
- *
- * function libxslt:test() for testing the extensions support.
- */
-static void
-xsltExtFunctionTest(xmlXPathParserContextPtr ctxt, int nargs)
-{
-    xsltTransformContextPtr tctxt;
-    void *data;
-
-    if (testData == NULL) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
-                         "xsltExtFunctionTest: not initialized\n");
-        return;
-    }
-    tctxt = xsltXPathGetTransformContext(ctxt);
-    if (tctxt == NULL) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
-                         "xsltExtFunctionTest: failed to get the transformation context\n");
-        return;
-    }
-    data = xsltGetExtData(tctxt, (const xmlChar *) XSLT_DEFAULT_URL);
-    if (data == NULL) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
-                         "xsltExtFunctionTest: failed to get module data\n");
-        return;
-    }
-    if (data != testData) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
-                         "xsltExtFunctionTest: got wrong module data\n");
-        return;
-    }
-#ifdef WITH_XSLT_DEBUG_FUNCTION
-    xsltGenericDebug(xsltGenericDebugContext,
-                     "libxslt:test() called with %d args\n", nargs);
-#endif
-}
-
-/**
- * xsltExtElementTest:
- * @ctxt:  an XSLT processing context
- * @node:  The current node
- * @inst:  the instruction in the stylesheet
- * @comp:  precomputed informations
- *
- * Process a libxslt:test node
- */
-static void
-xsltExtElementTest(xsltTransformContextPtr ctxt, xmlNodePtr node,
-                   xmlNodePtr inst,
-                   xsltStylePreCompPtr comp ATTRIBUTE_UNUSED)
-{
-    xmlNodePtr comment;
-
-    if (testData == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, inst);
-        xsltGenericError(xsltGenericErrorContext,
-                         "xsltExtElementTest: not initialized\n");
-        return;
-    }
-    if (ctxt == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, inst);
-        xsltGenericError(xsltGenericErrorContext,
-                         "xsltExtElementTest: no transformation context\n");
-        return;
-    }
-    if (node == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, inst);
-        xsltGenericError(xsltGenericErrorContext,
-                         "xsltExtElementTest: no current node\n");
-        return;
-    }
-    if (inst == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, inst);
-        xsltGenericError(xsltGenericErrorContext,
-                         "xsltExtElementTest: no instruction\n");
-        return;
-    }
-    if (ctxt->insert == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, inst);
-        xsltGenericError(xsltGenericErrorContext,
-                         "xsltExtElementTest: no insertion point\n");
-        return;
-    }
-    comment =
-        xmlNewComment((const xmlChar *)
-                      "libxslt:test element test worked");
-    xmlAddChild(ctxt->insert, comment);
-}
-
-/**
- * xsltExtInitTest:
- * @ctxt:  an XSLT transformation context
- * @URI:  the namespace URI for the extension
- *
- * A function called at initialization time of an XSLT extension module
- *
- * Returns a pointer to the module specific data for this transformation
- */
-static void *
-xsltExtInitTest(xsltTransformContextPtr ctxt, const xmlChar * URI)
-{
-    if (testData != NULL) {
-	xsltPrintErrorContext(ctxt, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
-                         "xsltExtInitTest: already initialized\n");
-        return (NULL);
-    }
-    testData = (void *) "test data";
-    xsltRegisterExtFunction(ctxt, (const xmlChar *) "test",
-                            (const xmlChar *) XSLT_DEFAULT_URL,
-                            xsltExtFunctionTest);
-    xsltRegisterExtElement(ctxt, (const xmlChar *) "test",
-                            (const xmlChar *) XSLT_DEFAULT_URL,
-                            xsltExtElementTest);
-
-    xsltGenericDebug(xsltGenericDebugContext,
-                     "Registered test module : %s\n", URI);
-    return (testData);
-}
-
-
-/**
- * xsltExtShutdownTest:
- * @ctxt:  an XSLT transformation context
- * @URI:  the namespace URI for the extension
- * @data:  the data associated to this module
- *
- * A function called at shutdown time of an XSLT extension module
- */
-static void
-xsltExtShutdownTest(xsltTransformContextPtr ctxt ATTRIBUTE_UNUSED,
-                    const xmlChar * URI, void *data)
-{
-    if (testData == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
-                         "xsltExtShutdownTest: not initialized\n");
-        return;
-    }
-    if (data != testData) {
-	xsltPrintErrorContext(ctxt, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
-                         "xsltExtShutdownTest: wrong data\n");
-    }
-    testData = NULL;
-    xsltGenericDebug(xsltGenericDebugContext,
-                     "Unregistered test module : %s\n", URI);
-}
-
 /************************************************************************
  * 									*
  * 		Registration of XSLT and libxslt functions		*
@@ -932,8 +786,4 @@ xsltRegisterAllFunctions(xmlXPathContextPtr ctxt)
                          xsltElementAvailableFunction);
     xmlXPathRegisterFunc(ctxt, (const xmlChar *) "function-available",
                          xsltFunctionAvailableFunction);
-
-    xsltRegisterExtModule((const xmlChar *) XSLT_DEFAULT_URL,
-	                  xsltExtInitTest,
-                          xsltExtShutdownTest);
 }
