@@ -125,8 +125,7 @@ xsltNewCompMatch(void) {
 
     cur = (xsltCompMatchPtr) xmlMalloc(sizeof(xsltCompMatch));
     if (cur == NULL) {
-	xsltPrintErrorContext(NULL, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, NULL, NULL,
 		"xsltNewCompMatch : malloc failed\n");
 	return(NULL);
     }
@@ -205,8 +204,7 @@ xsltNewParserContext(xsltStylesheetPtr style, xsltTransformContextPtr ctxt) {
 
     cur = (xsltParserContextPtr) xmlMalloc(sizeof(xsltParserContext));
     if (cur == NULL) {
-	xsltPrintErrorContext(NULL, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, NULL, NULL,
 		"xsltNewParserContext : malloc failed\n");
 	return(NULL);
     }
@@ -246,8 +244,7 @@ xsltCompMatchAdd(xsltParserContextPtr ctxt, xsltCompMatchPtr comp,
                  xsltOp op, xmlChar * value, xmlChar * value2)
 {
     if (comp->nbStep >= 40) {
-        xsltPrintErrorContext(NULL, NULL, NULL);        /* TODO */
-        xsltGenericError(xsltGenericErrorContext,
+        xsltTransformError(NULL, NULL, NULL,
                          "xsltCompMatchAdd: overflow\n");
         return (-1);
     }
@@ -366,8 +363,7 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
     xsltStepOpPtr step, select = NULL;
 
     if ((comp == NULL) || (node == NULL) || (ctxt == NULL)) {
-	xsltPrintErrorContext(ctxt, NULL, node);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, node,
 		"xsltTestCompMatch: null arg\n");
         return(-1);
     }
@@ -606,7 +602,7 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 		    index = (int)
 			XSLT_RUNTIME_EXTRA(ctxt, select->indexExtra);
 		    list = (xmlXPathObjectPtr)
-			XSLT_RUNTIME_EXTRA(ctxt, select->lenExtra);
+			XSLT_RUNTIME_EXTRA_LST(ctxt, select->lenExtra);
 		    
 		    doc = node->doc;
 		    if ((list == NULL) || (prevdoc != doc)) {
@@ -644,7 +640,7 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 				xmlXPathFreeObject(list);
 			    list = newlist;
 
-			    XSLT_RUNTIME_EXTRA(ctxt, select->lenExtra) =
+			    XSLT_RUNTIME_EXTRA_LST(ctxt, select->lenExtra) =
 				(void *) list;
 			    XSLT_RUNTIME_EXTRA(ctxt, select->previousExtra) =
 				(void *) doc;
@@ -939,17 +935,11 @@ wrong_index:
 		break;
             case XSLT_OP_NODE:
 		switch (node->type) {
-		    case XML_DOCUMENT_NODE:
-		    case XML_HTML_DOCUMENT_NODE:
-#ifdef LIBXML_DOCB_ENABLED
-		    case XML_DOCB_DOCUMENT_NODE:
-#endif
 		    case XML_ELEMENT_NODE:
 		    case XML_CDATA_SECTION_NODE:
 		    case XML_PI_NODE:
 		    case XML_COMMENT_NODE:
 		    case XML_TEXT_NODE:
-		    case XML_ATTRIBUTE_NODE:
 			break;
 		    default:
 			return(0);
@@ -1198,8 +1188,7 @@ xsltCompileIdKeyPattern(xsltParserContextPtr ctxt, xmlChar *name, int aid) {
     xmlChar *lit2 = NULL;
 
     if (CUR != '(') {
-	xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, NULL, NULL,
 		"xsltCompileIdKeyPattern : ( expected\n");
 	ctxt->error = 1;
 	return;
@@ -1212,8 +1201,7 @@ xsltCompileIdKeyPattern(xsltParserContextPtr ctxt, xmlChar *name, int aid) {
 	    return;
 	SKIP_BLANKS;
 	if (CUR != ')') {
-	    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, NULL, NULL,
 		    "xsltCompileIdKeyPattern : ) expected\n");
 	    ctxt->error = 1;
 	    return;
@@ -1228,8 +1216,7 @@ xsltCompileIdKeyPattern(xsltParserContextPtr ctxt, xmlChar *name, int aid) {
 	    return;
 	SKIP_BLANKS;
 	if (CUR != ',') {
-	    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, NULL, NULL,
 		    "xsltCompileIdKeyPattern : , expected\n");
 	    ctxt->error = 1;
 	    return;
@@ -1241,8 +1228,7 @@ xsltCompileIdKeyPattern(xsltParserContextPtr ctxt, xmlChar *name, int aid) {
 	    return;
 	SKIP_BLANKS;
 	if (CUR != ')') {
-	    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, NULL, NULL,
 		    "xsltCompileIdKeyPattern : ) expected\n");
 	    ctxt->error = 1;
 	    return;
@@ -1259,8 +1245,7 @@ xsltCompileIdKeyPattern(xsltParserContextPtr ctxt, xmlChar *name, int aid) {
 		return;
 	    SKIP_BLANKS;
 	    if (CUR != ')') {
-		xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-		xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(NULL, NULL, NULL,
 			"xsltCompileIdKeyPattern : ) expected\n");
 		ctxt->error = 1;
 		return;
@@ -1272,8 +1257,7 @@ xsltCompileIdKeyPattern(xsltParserContextPtr ctxt, xmlChar *name, int aid) {
 	NEXT;
 	SKIP_BLANKS;
 	if (CUR != ')') {
-	    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, NULL, NULL,
 		    "xsltCompileIdKeyPattern : ) expected\n");
 	    ctxt->error = 1;
 	    return;
@@ -1284,8 +1268,7 @@ xsltCompileIdKeyPattern(xsltParserContextPtr ctxt, xmlChar *name, int aid) {
 	NEXT;
 	SKIP_BLANKS;
 	if (CUR != ')') {
-	    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, NULL, NULL,
 		    "xsltCompileIdKeyPattern : ) expected\n");
 	    ctxt->error = 1;
 	    return;
@@ -1296,8 +1279,7 @@ xsltCompileIdKeyPattern(xsltParserContextPtr ctxt, xmlChar *name, int aid) {
 	NEXT;
 	SKIP_BLANKS;
 	if (CUR != ')') {
-	    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, NULL, NULL,
 		    "xsltCompileIdKeyPattern : ) expected\n");
 	    ctxt->error = 1;
 	    return;
@@ -1305,14 +1287,12 @@ xsltCompileIdKeyPattern(xsltParserContextPtr ctxt, xmlChar *name, int aid) {
 	NEXT;
 	PUSH(XSLT_OP_NODE, NULL, NULL);
     } else if (aid) {
-	xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, NULL, NULL,
 	    "xsltCompileIdKeyPattern : expecting 'key' or 'id' or node type\n");
 	ctxt->error = 1;
 	return;
     } else {
-	xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, NULL, NULL,
 	    "xsltCompileIdKeyPattern : node type\n");
 	ctxt->error = 1;
 	return;
@@ -1366,8 +1346,7 @@ xsltCompileStepPattern(xsltParserContextPtr ctxt, xmlChar *token) {
 
 	    ns = xmlSearchNs(ctxt->doc, ctxt->elem, prefix);
 	    if (ns == NULL) {
-		xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-		xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(NULL, NULL, NULL,
 		"xsltCompileStepPattern : no namespace bound to prefix %s\n",
 				 prefix);
 	    } else {
@@ -1381,8 +1360,7 @@ xsltCompileStepPattern(xsltParserContextPtr ctxt, xmlChar *token) {
 		PUSH(XSLT_OP_ATTR, NULL, URL);
 		return;
 	    }
-	    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, NULL, NULL,
 		    "xsltCompileStepPattern : Name expected\n");
 	    ctxt->error = 1;
 	    goto error;
@@ -1398,8 +1376,7 @@ xsltCompileStepPattern(xsltParserContextPtr ctxt, xmlChar *token) {
 	    PUSH(XSLT_OP_ALL, token, NULL);
 	    goto parse_predicate;
 	} else {
-	    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, NULL, NULL,
 		    "xsltCompileStepPattern : Name expected\n");
 	    ctxt->error = 1;
 	    goto error;
@@ -1424,8 +1401,7 @@ xsltCompileStepPattern(xsltParserContextPtr ctxt, xmlChar *token) {
 	    token = xsltScanName(ctxt);
 	    ns = xmlSearchNs(ctxt->doc, ctxt->elem, prefix);
 	    if (ns == NULL) {
-		xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-		xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(NULL, NULL, NULL,
 	    "xsltCompileStepPattern : no namespace bound to prefix %s\n",
 				 prefix);
 		ctxt->error = 1;
@@ -1439,8 +1415,7 @@ xsltCompileStepPattern(xsltParserContextPtr ctxt, xmlChar *token) {
 		    NEXT;
 		    PUSH(XSLT_OP_NS, URL, NULL);
 		} else {
-		    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-		    xsltGenericError(xsltGenericErrorContext,
+		    xsltTransformError(NULL, NULL, NULL,
 			    "xsltCompileStepPattern : Name expected\n");
 		    ctxt->error = 1;
 		    goto error;
@@ -1459,8 +1434,7 @@ xsltCompileStepPattern(xsltParserContextPtr ctxt, xmlChar *token) {
 	                PUSH(XSLT_OP_ALL, token, NULL);
 	                goto parse_predicate;
 	            } else {
-		        xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-		        xsltGenericError(xsltGenericErrorContext,
+		        xsltTransformError(NULL, NULL, NULL,
 			    "xsltCompileStepPattern : QName expected\n");
 		        ctxt->error = 1;
 		        goto error;
@@ -1480,8 +1454,7 @@ xsltCompileStepPattern(xsltParserContextPtr ctxt, xmlChar *token) {
 		xmlFree(token);
 		token = xsltScanName(ctxt);
 		if (token == NULL) {
-		    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-		    xsltGenericError(xsltGenericErrorContext,
+		    xsltTransformError(NULL, NULL, NULL,
 			    "xsltCompileStepPattern : QName expected\n");
 		    ctxt->error = 1;
 		    goto error;
@@ -1497,8 +1470,7 @@ xsltCompileStepPattern(xsltParserContextPtr ctxt, xmlChar *token) {
 		}
 		PUSH(XSLT_OP_ATTR, name, URL);
 	    } else {
-		xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-		xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(NULL, NULL, NULL,
 		    "xsltCompileStepPattern : 'child' or 'attribute' expected\n");
 		ctxt->error = 1;
 		goto error;
@@ -1548,8 +1520,7 @@ parse_predicate:
 	    NEXT;
 	}
 	if (CUR == 0) {
-	    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, NULL, NULL,
 		    "xsltCompileStepPattern : ']' expected\n");
 	    ctxt->error = 1;
 	    goto error;
@@ -1653,8 +1624,7 @@ xsltCompileLocationPathPattern(xsltParserContextPtr ctxt) {
 	xmlChar *name;
 	name = xsltScanName(ctxt);
 	if (name == NULL) {
-	    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, NULL, NULL,
 		    "xsltCompileLocationPathPattern : Name expected\n");
 	    ctxt->error = 1;
 	    return;
@@ -1707,8 +1677,7 @@ xsltCompilePattern(const xmlChar *pattern, xmlDocPtr doc,
     int current, start, end, level, j;
 
     if (pattern == NULL) {
-	xsltPrintErrorContext(NULL, NULL, node); /* TODO */
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, NULL, node,
 			 "xsltCompilePattern : NULL pattern\n");
 	return(NULL);
     }
@@ -1742,8 +1711,7 @@ xsltCompilePattern(const xmlChar *pattern, xmlDocPtr doc,
 	    end++;
 	}
 	if (current == end) {
-	    xsltPrintErrorContext(NULL, NULL, node); /* TODO */
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, NULL, node,
 			     "xsltCompilePattern : NULL pattern\n");
 	    goto error;
 	}
@@ -1779,8 +1747,7 @@ xsltCompilePattern(const xmlChar *pattern, xmlDocPtr doc,
 #endif
 	xsltCompileLocationPathPattern(ctxt);
 	if (ctxt->error) {
-	    xsltPrintErrorContext(NULL, style, node);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, node,
 			     "xsltCompilePattern : failed to compile '%s'\n",
 			     element->pattern);
 	    style->errors++;
@@ -1842,8 +1809,7 @@ xsltCompilePattern(const xmlChar *pattern, xmlDocPtr doc,
 	current = end;
     }
     if (end == 0) {
-	xsltPrintErrorContext(NULL, NULL, node); /* TODO */
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, node,
 			 "xsltCompilePattern : NULL pattern\n");
 	style->errors++;
 	goto error;
@@ -1931,8 +1897,7 @@ xsltAddTemplate(xsltStylesheetPtr style, xsltTemplatePtr cur,
 	    break;
         case XSLT_OP_END:
 	case XSLT_OP_PREDICATE:
-	    xsltPrintErrorContext(NULL, style, NULL);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, NULL,
 			     "xsltAddTemplate: invalid compiled pattern\n");
 	    xsltFreeCompMatch(pat);
 	    return(-1);
@@ -2013,8 +1978,7 @@ xsltAddTemplate(xsltStylesheetPtr style, xsltTemplatePtr cur,
 		list->next = pat;
 	    }
 	} else {
-	    xsltPrintErrorContext(NULL, style, NULL);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, NULL,
 			     "xsltAddTemplate: invalid compiled pattern\n");
 	    xsltFreeCompMatch(pat);
 	    return(-1);
@@ -2174,7 +2138,8 @@ xsltGetTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	 * Some of the tests for elements can also apply to documents
 	 */
 	if ((node->type == XML_DOCUMENT_NODE) ||
-	    (node->type == XML_HTML_DOCUMENT_NODE)) {
+	    (node->type == XML_HTML_DOCUMENT_NODE) ||
+	    (node->type == XML_TEXT_NODE)) {
 	    list = curstyle->elemMatch;
 	    while ((list != NULL) &&
 		   ((ret == NULL)  || (list->priority > priority))) {
