@@ -53,8 +53,7 @@ xsltNewStackElem(void) {
 
     cur = (xsltStackElemPtr) xmlMalloc(sizeof(xsltStackElem));
     if (cur == NULL) {
-	xsltPrintErrorContext(NULL, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, NULL, NULL,
 		"xsltNewStackElem : malloc failed\n");
 	return(NULL);
     }
@@ -82,8 +81,7 @@ xsltCopyStackElem(xsltStackElemPtr elem) {
 
     cur = (xsltStackElemPtr) xmlMalloc(sizeof(xsltStackElem));
     if (cur == NULL) {
-	xsltPrintErrorContext(NULL, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, NULL, NULL,
 		"xsltCopyStackElem : malloc failed\n");
 	return(NULL);
     }
@@ -144,7 +142,8 @@ xsltFreeStackElemList(xsltStackElemPtr elem) {
  *
  * check wether the variable or param is already defined
  *
- * Returns 1 if present, 0 if not, -1 in case of failure.
+ * Returns 1 if variable is present, 2 if param is present, 3 if this
+ *         is an inherited param, 0 if not found, -1 in case of failure.
  */
 static int
 xsltCheckStackElem(xsltTransformContextPtr ctxt, const xmlChar *name,
@@ -160,6 +159,12 @@ xsltCheckStackElem(xsltTransformContextPtr ctxt, const xmlChar *name,
 	    if (((nameURI == NULL) && (cur->nameURI == NULL)) ||
 		((nameURI != NULL) && (cur->nameURI != NULL) &&
 		 (xmlStrEqual(nameURI, cur->nameURI)))) {
+		if ((cur->comp != NULL) &&
+		    (cur->comp->type == XSLT_FUNC_WITHPARAM))
+		    return(3);
+		if ((cur->comp != NULL) &&
+		    (cur->comp->type == XSLT_FUNC_PARAM))
+		    return(2);
 		return(1);
 	    }
 	}
@@ -335,8 +340,7 @@ xsltEvalVariable(xsltTransformContextPtr ctxt, xsltStackElemPtr elem,
 	if ((precomp == NULL) || (precomp->comp == NULL))
 	    xmlXPathFreeCompExpr(comp);
 	if (result == NULL) {
-	    xsltPrintErrorContext(ctxt, NULL, precomp->inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(ctxt, NULL, precomp->inst,
 		"Evaluating variable %s failed\n", elem->name);
 	    ctxt->state = XSLT_STATE_STOPPED;
 #ifdef WITH_XSLT_DEBUG_VARIABLE
@@ -466,8 +470,7 @@ xsltEvalGlobalVariable(xsltStackElemPtr elem, xsltTransformContextPtr ctxt) {
 	if ((precomp == NULL) || (precomp->comp == NULL))
 	    xmlXPathFreeCompExpr(comp);
 	if (result == NULL) {
-	    xsltPrintErrorContext(ctxt, NULL, precomp->inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(ctxt, NULL, precomp->inst,
 		"Evaluating global variable %s failed\n", elem->name);
 	    ctxt->state = XSLT_STATE_STOPPED;
 #ifdef WITH_XSLT_DEBUG_VARIABLE
@@ -595,8 +598,7 @@ xsltEvalGlobalVariables(xsltTransformContextPtr ctxt) {
 		if ((elem->comp->inst != NULL) &&
 		    (def->comp != NULL) && (def->comp->inst != NULL) &&
 		    (elem->comp->inst->doc == def->comp->inst->doc)) {
-		    xsltPrintErrorContext(ctxt, style, elem->comp->inst);
-		    xsltGenericError(xsltGenericErrorContext,
+		    xsltTransformError(ctxt, style, elem->comp->inst,
 			"Global variable %s already defined\n", elem->name);
 		}
 	    }
@@ -759,8 +761,7 @@ xsltProcessUserParamInternal(xsltTransformContextPtr ctxt,
 	    ns = xmlSearchNs(style->doc, xmlDocGetRootElement(style->doc),
 			     prefix);
 	    if (ns == NULL) {
-		xsltPrintErrorContext(ctxt, style, NULL);
-		xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(ctxt, style, NULL,
 		"user param : no namespace bound to prefix %s\n", prefix);
 		href = NULL;
 	    } else {
@@ -810,8 +811,7 @@ xsltProcessUserParamInternal(xsltTransformContextPtr ctxt,
 	    xmlXPathFreeCompExpr(comp);
 	}
 	if (result == NULL) {
-	    xsltPrintErrorContext(ctxt, style, NULL);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(ctxt, style, NULL,
 		"Evaluating user parameter %s failed\n", name);
 	    ctxt->state = XSLT_STATE_STOPPED;
 	    xmlFree(ncname);
@@ -865,8 +865,7 @@ xsltProcessUserParamInternal(xsltTransformContextPtr ctxt,
     res = xmlHashAddEntry2(ctxt->globalVars, ncname, href, elem);
     if (res != 0) {
 	xsltFreeStackElem(elem);
-	xsltPrintErrorContext(ctxt, style, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, style, NULL,
 	    "Global parameter %s already defined\n", ncname);
     }
     xmlFree(ncname);
@@ -1043,17 +1042,24 @@ static int
 xsltRegisterVariable(xsltTransformContextPtr ctxt, xsltStylePreCompPtr comp,
 		     xmlNodePtr tree, int param) {
     xsltStackElemPtr elem;
+    int present;
 
-    if (xsltCheckStackElem(ctxt, comp->name, comp->ns) != 0) {
-	if (!param) {
-	    xsltPrintErrorContext(ctxt, NULL, comp->inst);
-	    xsltGenericError(xsltGenericErrorContext,
-	    "xsl:variable : redefining %s\n", comp->name);
+    present = xsltCheckStackElem(ctxt, comp->name, comp->ns);
+    if (param == 0) {
+	if ((present != 0) && (present != 3)) {
+	    xsltTransformError(ctxt, NULL, comp->inst,
+		"xsl:variable : redefining %s\n", comp->name);
+	    return(0);
+	}
+    } else if (present != 0) {
+	if ((present == 1) || (present == 2)) {
+	    xsltTransformError(ctxt, NULL, comp->inst,
+		"xsl:param : redefining %s\n", comp->name);
+	    return(0);
 	}
 #ifdef WITH_XSLT_DEBUG_VARIABLE
-	else
-	    xsltGenericDebug(xsltGenericDebugContext,
-		     "param %s defined by caller\n", comp->name);
+	xsltGenericDebug(xsltGenericDebugContext,
+		 "param %s defined by caller\n", comp->name);
 #endif
 	return(0);
     }
@@ -1161,15 +1167,13 @@ xsltParseStylesheetCallerParam(xsltTransformContextPtr ctxt, xmlNodePtr cur) {
 	return(NULL);
     comp = (xsltStylePreCompPtr) cur->_private;
     if (comp == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, cur,
 	    "xsl:param : compilation error\n");
 	return(NULL);
     }
 
     if (comp->name == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, cur,
 	    "xsl:param : missing name attribute\n");
 	return(NULL);
     }
@@ -1213,15 +1217,13 @@ xsltParseGlobalVariable(xsltStylesheetPtr style, xmlNodePtr cur) {
     xsltStylePreCompute(style, cur);
     comp = (xsltStylePreCompPtr) cur->_private;
     if (comp == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	     "xsl:variable : compilation failed\n");
 	return;
     }
 
     if (comp->name == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	    "xsl:variable : missing name attribute\n");
 	return;
     }
@@ -1254,15 +1256,13 @@ xsltParseGlobalParam(xsltStylesheetPtr style, xmlNodePtr cur) {
     xsltStylePreCompute(style, cur);
     comp = (xsltStylePreCompPtr) cur->_private;
     if (comp == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	     "xsl:param : compilation failed\n");
 	return;
     }
 
     if (comp->name == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	    "xsl:param : missing name attribute\n");
 	return;
     }
@@ -1294,15 +1294,13 @@ xsltParseStylesheetVariable(xsltTransformContextPtr ctxt, xmlNodePtr cur) {
 
     comp = (xsltStylePreCompPtr) cur->_private;
     if (comp == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, cur,
 	     "xsl:variable : compilation failed\n");
 	return;
     }
 
     if (comp->name == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, cur,
 	    "xsl:variable : missing name attribute\n");
 	return;
     }
@@ -1333,15 +1331,13 @@ xsltParseStylesheetParam(xsltTransformContextPtr ctxt, xmlNodePtr cur) {
 
     comp = (xsltStylePreCompPtr) cur->_private;
     if (comp == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, cur,
 	     "xsl:param : compilation failed\n");
 	return;
     }
 
     if (comp->name == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, cur,
 	    "xsl:param : missing name attribute\n");
 	return;
     }
@@ -1394,8 +1390,7 @@ xsltXPathVariableLookup(void *ctxt, const xmlChar *name,
     context = (xsltTransformContextPtr) ctxt;
     ret = xsltVariableLookup(context, name, ns_uri);
     if (ret == NULL) {
-	xsltPrintErrorContext(ctxt, NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(ctxt, NULL, NULL,
 	    "unregistered variable %s\n", name);
     }
 #ifdef WITH_XSLT_DEBUG_VARIABLE
