@@ -40,6 +40,7 @@
 #include "extensions.h"
 #include "preproc.h"
 #include "extra.h"
+#include "security.h"
 
 #ifdef WITH_XSLT_DEBUG
 #define WITH_XSLT_DEBUG_PARSING
@@ -271,8 +272,7 @@ xsltNewTemplate(void) {
 
     cur = (xsltTemplatePtr) xmlMalloc(sizeof(xsltTemplate));
     if (cur == NULL) {
-	xsltPrintErrorContext(NULL, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, NULL, NULL,
 		"xsltNewTemplate : malloc failed\n");
 	return(NULL);
     }
@@ -331,8 +331,7 @@ xsltNewStylesheet(void) {
 
     cur = (xsltStylesheetPtr) xmlMalloc(sizeof(xsltStylesheet));
     if (cur == NULL) {
-	xsltPrintErrorContext(NULL, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, NULL, NULL,
 		"xsltNewStylesheet : malloc failed\n");
 	return(NULL);
     }
@@ -644,8 +643,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
                 (xmlStrEqual(prop, (const xmlChar *) "text"))) {
                 style->method = prop;
             } else {
-		xsltPrintErrorContext(NULL, style, cur);
-                xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(NULL, style, cur,
                                  "invalid value for method: %s\n", prop);
                 style->warnings++;
             }
@@ -681,8 +679,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
         } else if (xmlStrEqual(prop, (const xmlChar *) "no")) {
             style->standalone = 0;
         } else {
-	    xsltPrintErrorContext(NULL, style, cur);
-            xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
                              "invalid value for standalone: %s\n", prop);
             style->warnings++;
         }
@@ -696,8 +693,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
         } else if (xmlStrEqual(prop, (const xmlChar *) "no")) {
             style->indent = 0;
         } else {
-	    xsltPrintErrorContext(NULL, style, cur);
-            xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
                              "invalid value for indent: %s\n", prop);
             style->warnings++;
         }
@@ -712,8 +708,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
         } else if (xmlStrEqual(prop, (const xmlChar *) "no")) {
             style->omitXmlDeclaration = 0;
         } else {
-	    xsltPrintErrorContext(NULL, style, cur);
-            xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
                              "invalid value for omit-xml-declaration: %s\n",
                              prop);
             style->warnings++;
@@ -761,6 +756,13 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
         }
         xmlFree(elements);
     }
+
+    prop = xsltGetNsProp(cur, (const xmlChar *) "media-type", XSLT_NAMESPACE);
+    if (prop != NULL) {
+	if (style->mediaType)
+	    xmlFree(style->mediaType);
+	style->mediaType = prop;
+    }
 }
 
 /**
@@ -787,16 +789,14 @@ xsltParseStylesheetDecimalFormat(xsltStylesheetPtr style, xmlNodePtr cur)
     if (prop != NULL) {
 	format = xsltDecimalFormatGetByName(style, prop);
 	if (format != NULL) {
-	    xsltPrintErrorContext(NULL, style, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
 	 "xsltParseStylestyleDecimalFormat: %s already exists\n", prop);
 	    style->warnings++;
 	    return;
 	}
 	format = xsltNewDecimalFormat(prop);
 	if (format == NULL) {
-	    xsltPrintErrorContext(NULL, style, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
      "xsltParseStylestyleDecimalFormat: failed creating new decimal-format\n");
 	    style->errors++;
 	    return;
@@ -891,8 +891,7 @@ xsltParseStylesheetPreserveSpace(xsltStylesheetPtr style, xmlNodePtr cur) {
 
     elements = xsltGetNsProp(cur, (const xmlChar *)"elements", XSLT_NAMESPACE);
     if (elements == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	    "xsltParseStylesheetPreserveSpace: missing elements attribute\n");
 	style->warnings++;
 	return;
@@ -972,8 +971,7 @@ xsltParseStylesheetExtPrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
 	    else
 		ns = xmlSearchNs(style->doc, cur, prefix);
 	    if (ns == NULL) {
-		xsltPrintErrorContext(NULL, style, cur);
-		xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(NULL, style, cur,
 	    "xsl:extension-element-prefix : undefined namespace %s\n",
 	                         prefix);
 		style->warnings++;
@@ -1010,8 +1008,7 @@ xsltParseStylesheetStripSpace(xsltStylesheetPtr style, xmlNodePtr cur) {
 
     elements = xsltGetNsProp(cur, (const xmlChar *)"elements", XSLT_NAMESPACE);
     if (elements == NULL) {
-	xsltPrintErrorContext(NULL, style, cur);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, cur,
 	    "xsltParseStylesheetStripSpace: missing elements attribute\n");
 	style->warnings++;
 	return;
@@ -1094,8 +1091,7 @@ xsltParseStylesheetExcludePrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
 	    else
 		ns = xmlSearchNs(style->doc, cur, prefix);
 	    if (ns == NULL) {
-		xsltPrintErrorContext(NULL, style, cur);
-		xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(NULL, style, cur,
 	    "xsl:exclude-result-prefixes : undefined namespace %s\n",
 	                         prefix);
 		style->warnings++;
@@ -1288,8 +1284,7 @@ xsltGatherNamespaces(xsltStylesheetPtr style) {
 		    if (style->nsHash == NULL) {
 			style->nsHash = xmlHashCreate(10);
 			if (style->nsHash == NULL) {
-			    xsltPrintErrorContext(NULL, style, cur);
-			    xsltGenericError(xsltGenericErrorContext,
+			    xsltTransformError(NULL, style, cur,
 		 "xsltGatherNamespaces: failed to create hash table\n");
 			    style->errors++;
 			    return;
@@ -1297,8 +1292,7 @@ xsltGatherNamespaces(xsltStylesheetPtr style) {
 		    }
 		    URI = xmlHashLookup(style->nsHash, ns->prefix);
 		    if ((URI != NULL) && (!xmlStrEqual(URI, ns->href))) {
-			xsltPrintErrorContext(NULL, style, cur);
-			xsltGenericError(xsltGenericErrorContext,
+			xsltTransformError(NULL, style, cur,
 	     "Namespaces prefix %s used for multiple namespaces\n");
 			style->warnings++;
 		    } else if (URI == NULL) {
@@ -1394,8 +1388,7 @@ xsltParseTemplateContent(xsltStylesheetPtr style, xmlNodePtr templ) {
 			    noesc = 1;
 			} else if (!xmlStrEqual(prop,
 						(const xmlChar *)"no")){
-			    xsltPrintErrorContext(NULL, style, cur);
-			    xsltGenericError(xsltGenericErrorContext,
+			    xsltTransformError(NULL, style, cur,
 	     "xsl:text: disable-output-escaping allows only yes or no\n");
 			    style->warnings++;
 
@@ -1406,8 +1399,7 @@ xsltParseTemplateContent(xsltStylesheetPtr style, xmlNodePtr templ) {
 		    while (text != NULL) {
 			if ((text->type != XML_TEXT_NODE) &&
 			     (text->type != XML_CDATA_SECTION_NODE)) {
-			    xsltPrintErrorContext(NULL, style, cur);
-			    xsltGenericError(xsltGenericErrorContext,
+			    xsltTransformError(NULL, style, cur,
 		 "xsltParseTemplateContent: xslt:text content problem\n");
 			    style->errors++;
 			    break;
@@ -1499,8 +1491,7 @@ skip_children:
 	    xmlNodePtr param = cur;
 
             cur = cur->next;
-	    xsltPrintErrorContext(NULL, style, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
 		"xsltParseTemplateContent: ignoring misplaced param element\n");
 	    style->warnings++;
 	    xmlUnlinkNode(param);
@@ -1552,8 +1543,7 @@ xsltParseStylesheetKey(xsltStylesheetPtr style, xmlNodePtr key) {
 	     "xsltParseStylesheetKey: name %s\n", name);
 #endif
     } else {
-	xsltPrintErrorContext(NULL, style, key);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, key,
 	    "xsl:key : error missing name\n");
 	style->errors++;
 	goto error;
@@ -1561,8 +1551,7 @@ xsltParseStylesheetKey(xsltStylesheetPtr style, xmlNodePtr key) {
 
     match = xsltGetNsProp(key, (const xmlChar *)"match", XSLT_NAMESPACE);
     if (match == NULL) {
-	xsltPrintErrorContext(NULL, style, key);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, key,
 	    "xsl:key : error missing match\n");
 	style->errors++;
 	goto error;
@@ -1570,8 +1559,7 @@ xsltParseStylesheetKey(xsltStylesheetPtr style, xmlNodePtr key) {
 
     use = xsltGetNsProp(key, (const xmlChar *)"use", XSLT_NAMESPACE);
     if (use == NULL) {
-	xsltPrintErrorContext(NULL, style, key);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, key,
 	    "xsl:key : error missing use\n");
 	style->errors++;
 	goto error;
@@ -1729,15 +1717,13 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
 
     prop = xsltGetNsProp(top, (const xmlChar *)"version", XSLT_NAMESPACE);
     if (prop == NULL) {
-	xsltPrintErrorContext(NULL, style, top);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, top,
 	    "xsl:version is missing: document may not be a stylesheet\n");
 	style->warnings++;
     } else {
 	if ((!xmlStrEqual(prop, (const xmlChar *)"1.0")) &&
             (!xmlStrEqual(prop, (const xmlChar *)"1.1"))) {
-	    xsltPrintErrorContext(NULL, style, top);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, top,
 		"xsl:version: only 1.0 features are supported\n");
 	     /* TODO set up compatibility when not XSLT 1.0 */
 	    style->warnings++;
@@ -1775,8 +1761,7 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
 	}
 	if (cur->type == XML_TEXT_NODE) {
 	    if (cur->content != NULL) {
-		xsltPrintErrorContext(NULL, style, cur);
-		xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(NULL, style, cur,
 		    "misplaced text element: '%s'\n", cur->content);
 	    }
 	    style->errors++;
@@ -1808,8 +1793,7 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
 	    continue;
 	}
 	if (IS_XSLT_NAME(cur, "import")) {
-	    xsltPrintErrorContext(NULL, style, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
 			"xsltParseStylesheetTop: ignoring misplaced import element\n");
 	    style->errors++;
     } else if (IS_XSLT_NAME(cur, "include")) {
@@ -1839,8 +1823,7 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
     } else if (IS_XSLT_NAME(cur, "namespace-alias")) {
 	    xsltNamespaceAlias(style, cur);
 	} else {
-	    xsltPrintErrorContext(NULL, style, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
 			"xsltParseStylesheetTop: ignoring unknown %s element\n",
 			cur->name);
 	    style->warnings++;
@@ -1880,8 +1863,7 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
      */
     cur = xmlDocGetRootElement(doc);
     if (cur == NULL) {
-	xsltPrintErrorContext(NULL, ret, (xmlNodePtr) doc);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, ret, (xmlNodePtr) doc,
 		"xsltParseStylesheetProcess : empty stylesheet\n");
 	return(NULL);
     }
@@ -1906,8 +1888,7 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
 	 */
 	prop = xsltGetNsProp(cur, (const xmlChar *)"version", XSLT_NAMESPACE);
 	if (prop == NULL) {
-	    xsltPrintErrorContext(NULL, ret, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, ret, cur,
 		"xsltParseStylesheetProcess : document is not a stylesheet\n");
 	    return(NULL);
 	}
@@ -1918,8 +1899,7 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
 #endif
 	
 	if (!xmlStrEqual(prop, (const xmlChar *)"1.0")) {
-	    xsltPrintErrorContext(NULL, ret, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, ret, cur,
 		"xsl:version: only 1.0 features are supported\n");
 	     /* TODO set up compatibility when not XSLT 1.0 */
 	    ret->warnings++;
@@ -1992,6 +1972,7 @@ xsltParseStylesheetDoc(xmlDocPtr doc) {
 
 xsltStylesheetPtr
 xsltParseStylesheetFile(const xmlChar* filename) {
+    xsltSecurityPrefsPtr sec;
     xsltStylesheetPtr ret;
     xmlDocPtr doc;
     
@@ -2004,10 +1985,25 @@ xsltParseStylesheetFile(const xmlChar* filename) {
 	    "xsltParseStylesheetFile : parse %s\n", filename);
 #endif
 
+    /*
+     * Security framework check
+     */
+    sec = xsltGetDefaultSecurityPrefs();
+    if (sec != NULL) {
+	int res;
+
+	res = xsltCheckRead(sec, NULL, filename);
+	if (res == 0) {
+	    xsltTransformError(NULL, NULL, NULL,
+		 "xsltParseStylesheetFile: read rights for %s denied\n",
+			     filename);
+	    return(NULL);
+	}
+    }
+
     doc = xmlParseFile((const char *) filename);
     if (doc == NULL) {
-	xsltPrintErrorContext(NULL, NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, NULL, NULL,
 		"xsltParseStylesheetFile : cannot parse %s\n", filename);
 	return(NULL);
     }
@@ -2167,8 +2163,7 @@ xsltLoadStylesheetPI(xmlDocPtr doc) {
 #endif
 	URI = xmlParseURI((const char *) href);
 	if (URI == NULL) {
-	    xsltPrintErrorContext(NULL, NULL, child);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, NULL, child,
 		    "xml-stylesheet : href %s is not valid\n", href);
 	    xmlFree(href);
 	    return(NULL);
@@ -2188,8 +2183,7 @@ xsltLoadStylesheetPI(xmlDocPtr doc) {
 	    else
 		ID = xmlGetID(doc, (const xmlChar *) URI->fragment);
 	    if (ID == NULL) {
-		xsltPrintErrorContext(NULL, NULL, child);
-		xsltGenericError(xsltGenericErrorContext,
+		xsltTransformError(NULL, NULL, child,
 		    "xml-stylesheet : no ID %s found\n", URI->fragment);
 	    } else {
 		xmlDocPtr fake;
