@@ -455,6 +455,10 @@ struct _xsltTransformContext {
     xsltRuntimeExtraPtr extras;		/* extra per runtime informations */
 
     xsltDocumentPtr  styleList;		/* the stylesheet docs list */
+    void                 * sec;		/* the security preferences if any */
+
+    xmlGenericErrorFunc  error;		/* a specific error handler */
+    void              * errctx;		/* context for the error handler */
 };
 
 /**
